#include <iostream>
#include "3dengfx/3dengfx.hpp"
#include "common/timer.h"
#include "fxwt/fxwt.hpp"

bool Init();
void CleanUp();
void UpdateGfx();
void KeyHandler(int key);

Light *light;
TargetCamera *cam;
Object *plane, *torus;
ntimer timer;

Texture *env;

int main() {
	if(!Init()) {
		return -1;
	}

	return fxwt::MainLoop();
}

bool Init() {
	GraphicsInitParameters *gip;
	if(!(gip = LoadGraphicsContextConfig("3dengfx.conf"))) {
		return false;
	}

	if(!CreateGraphicsContext(*gip)) {
		return false;
	}

	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

	fxwt::SetWindowTitle("linux does it better(tm)");

	fxwt::SetDisplayHandler(UpdateGfx);
	fxwt::SetIdleHandler(UpdateGfx);
	fxwt::SetKeyboardHandler(KeyHandler);
	atexit(CleanUp);

	light = new PointLight(Vector3(50, 500, -100));
	cam = new TargetCamera(Vector3(0, 20, -50), Vector3(0, 0, 0));	
	
	plane = new Object;
	CreatePlane(plane->GetTriMeshPtr(), Plane(Vector3(0, 0, 0)), Vector2(50, 40), 16);
	plane->Rotate(Vector3(-half_pi, 0, 0));
	plane->Translate(Vector3(0, -5, 0));
	plane->GetMaterialPtr()->diffuse_color = Color(0.5f, 0.5f, 0.5f);
	plane->GetMaterialPtr()->SetTexture(GetTexture("test.png"), TEXTYPE_DIFFUSE);

	torus = new Object;
	CreateTorus(torus->GetTriMeshPtr(), 3.0, 10.0, 20);
	torus->Translate(Vector3(0, 5, 0));
	torus->GetMaterialPtr()->diffuse_color = Color(0.8, 0.2, 0.1);
	torus->GetMaterialPtr()->specular_color = Color(1.0, 1.0, 1.0);
	torus->GetMaterialPtr()->specular_power = 70.0;

	timer_reset(&timer);
	timer_start(&timer);
	return true;
}

void UpdateGfx() {
	float t = (float)timer_getmsec(&timer) / 1000.0f;
	
	cam->Activate();
	light->SetGLLight(0);
	
	Clear(0);
	ClearZBufferStencil(1.0f, 0);
	
	plane->Render();

	torus->SetRotation(Vector3(t, t, 0));
	torus->Render();
	
	Flip();
}

void CleanUp() {
	delete light;
	delete cam;
	delete plane;
	delete torus;
	SDL_EnableKeyRepeat(0, 0);
	DestroyGraphicsContext();
}

void KeyHandler(int key) {
	switch(key) {
	case 'a':
		cam->Zoom(0.95);
		break;

	case 'z':
		cam->Zoom(1.05);
		break;

	case '`':
		ScreenCapture();
		break;

	case SDLK_LEFT:
		cam->Rotate(Vector3(0.0, -0.05, 0.0));
		break;

	case SDLK_RIGHT:
		cam->Rotate(Vector3(0.0, 0.05, 0.0));
		break;

	case SDLK_UP:
		cam->Translate(Vector3(0.0, 0.5, 0.0));
		break;

	case SDLK_DOWN:
		cam->Translate(Vector3(0.0, -0.5, 0.0));
		break;

	case 27:
		exit(0);

	default:
		break;
	}
}
